@@ -1,3 +1,9 @@
+import tensorflow as tf
+# Prevent tensorflow from allocating whole GPU memory.
+gpus = tf.config.experimental.list_physical_devices('GPU')
+for gpu in gpus:
+  tf.config.experimental.set_memory_growth(gpu, True)
+
 from .dependency_parser import DependencyParser
 from .named_entity_recognizer import NamedEntityRecognizer
 from .normalizer import Normalizer
