@@ -2,6 +2,7 @@ from typing import List
 
 import pickle
 
+import tensorflow as tf
 import numpy as np
 
 from ..tokenizer import TreebankWordTokenize
@@ -62,7 +63,7 @@ class StemmerAnalyzer:
     - This version uses GRU instead of LSTM, which decreases the number of parameters by 25% with no actual performance penalty.
     - This version has an extra Dense layer before the output(p) layer.
     - During training, the positions of candidates and labels are shuffled in every batch.
-    - It achieves 0.9596 accuracy on ambigious tokens and 0.9745 accuracy on all tokens on trmorph2006 dataset, compared to 0.910 and 0.964 in the original paper.
+    - It achieves 0.9467 accuracy on ambigious tokens and 0.9664 accuracy on all tokens on trmorph2006 dataset, compared to 0.910 and 0.964 in the original paper.
     - For more details about the implementation, training procedure and evaluation metrics, see `ReadMe <https://github.com/vngrs-ai/VNLP/blob/main/vnlp/stemmer_morph_analyzer/ReadMe.md>`_.
     """
     def __init__(self, evaluate = False):
@@ -100,13 +101,15 @@ class StemmerAnalyzer:
         self.candidate_generator = TurkishStemSuffixCandidateGenerator(case_sensitive=True)
 
 
-    def predict(self, sentence: str) -> List[str]:
+    def predict(self, sentence: str, batch_size: int = 64) -> List[str]:
         """
         High level user API for Morphological Disambiguation.
 
         Args:
             sentence:
                 Input text(sentence).
+            batch_size:
+                batch size (number of tokens to be predicted together.) In case of long sentences, you can increase this for better performance on GPU. If you come across OOM error, decrease until error disappears.
 
         Returns:
             List of selected stem and morphological tags for each token.
@@ -151,19 +154,33 @@ class StemmerAnalyzer:
         ambig_levels = np.array([len(analyses) for analyses in sentence[1]])
 
         # Model Output (indices)
-        probs_of_sentence = self.model(X).numpy()
-        predicted_indices = []
-        for idx, probs_of_single_token in enumerate(probs_of_sentence):
-            ambig_level_of_token = ambig_levels[idx]
-            # Below code prevents model from giving output higher than the ambiguity level
-            # e.g. in an edge case, when ambiguity is 3, model returned 7.
-            # instead of argmax over all indices, now I return argmax of first ambig_level_of_token
-            # from output layer
-            probs_of_single_token = probs_of_single_token[:ambig_level_of_token]
-            predicted_indice_for_token = np.argmax(probs_of_single_token)
-            predicted_indices.append(predicted_indice_for_token)
+        if len(tokens) <= batch_size:
+            probs_of_sentence = self.model(X)
+        else:
+            probs_of_sentence = tf.constant(0, shape = (0, NUM_MAX_ANALYSIS), dtype = tf.float32)
+            for idx in range(0, len(tokens), batch_size):
+                low = idx
+                high = low + batch_size
+                probs_ = self.model([X_[low:high] for X_ in X])
+                probs_of_sentence = tf.concat((probs_of_sentence, probs_), axis = 0)
 
-        predicted_indices = np.array(predicted_indices)
+
+        # Below code prevents model from giving output higher than the ambiguity level
+        # e.g. in an edge case, when ambiguity is 3, model returned 7.
+        # instead of argmax over all indices, now I return argmax of first ambig_level_of_token
+        # from output layer
+        # -------------------------------------------------------------------------------
+        # Vectorized Implementation note:
+        # Instead of iterating over logits to exclude indices higher than ambiguity level
+        # I create a 2D mask and then argmax, which result in the same outcome
+        # This vectorized implementation is MUCH faster (680 ms vs 4 ms)
+        ambig_levels_tiled = tf.tile(tf.expand_dims(ambig_levels, axis = 1), [1, NUM_MAX_ANALYSIS])
+        tf_range = tf.range(1, NUM_MAX_ANALYSIS+1)
+        mask = tf.cast(ambig_levels_tiled >= tf_range, dtype = tf.float32)
+        probs_of_sentence  = probs_of_sentence * mask
+
+        # tf.argmax() --> numpy() is faster than numpy() --> np.argmax().
+        predicted_indices = tf.argmax(probs_of_sentence, axis = -1).numpy()
 
         # Obtaining Result
         result = []
